/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include <stdio.h>

#include "XLCoreP.h"
#include "../aladder.h"

static void ClassInitialize(void);
static void Initialize(Widget, Widget, ArgList args, Cardinal *num);
static void Destroy(Widget);
static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
static void Redisplay(Widget widget, XEvent *ev, Region region);
static void Resize(Widget widget);

static void ClickLeft(Widget w, XEvent *ev, String *p, Cardinal *n);
static void ClickRight(Widget w, XEvent *ev, String *p, Cardinal *n);
static void Motion(Widget w, XEvent *ev, String *p, Cardinal *n);
static void CursorUp(Widget widget, XEvent *ev, String *p, Cardinal *n);
static void CursorDown(Widget widget, XEvent *ev, String *p, Cardinal *n);

#define offset(field) XtOffsetOf(XLCoreRec, field)
static XtResource resources[] = {
    {XtNfont, XtCFont, XtRFontStruct, sizeof(XFontStruct *),
     offset(xlcore.font), XtRString, XtDefaultFont}
    ,
};

#define Win XtWindow(alCore.x11_data.core)
#define Disp XtDisplay(alCore.x11_data.core)

static XtActionsRec actions[] = {
    {"click_left", ClickLeft},
    {"click_right", ClickRight},
    {"motion", Motion},
    {"cursor_up", CursorUp},
    {"cursor_down", CursorDown},
};

static char translations[] =
    "<Motion>: motion()\n"
    "<Btn1Down>: click_left()\n"
    "<Btn3Down>: click_right()\n"
    "<Key>Return: click_left()\n"
    "<Key>Escape: click_right()\n"
    "<Key>Up: cursor_up()\n" "<Key>Down: cursor_down()\n";

#define superclass (&coreClassRec)
XLCoreClassRec xlCoreClassRec = {
    {
     /* superclass */ (WidgetClass) superclass,
     /* class_name */ "XLCore",
     /* size */ sizeof(XLCoreRec),
     /* class_initialize */ ClassInitialize,
     /* class_part_initialize */ NULL,
     /* Class init'ed */ FALSE,
     /* initialize */ Initialize,
     /* initialize_hook */ NULL,
     /* realize */ Realize,
     /* actions */ actions,
     /* num_actions */ XtNumber(actions),
     /* resources */ resources,
     /* resource_count */ XtNumber(resources),
     /* xrm_class */ NULLQUARK,
     /* compress_motion */ TRUE,
     /* compress_exposure */ TRUE,
     /* compress_enterleave */ TRUE,
     /* visible_interest */ FALSE,
     /* destroy */ Destroy,
     /* resize */ Resize,
     /* expose */ Redisplay,
     /* set_values */ NULL,
     /* set_values_hook */ NULL,
     /* set_values_almost */ XtInheritSetValuesAlmost,
     /* get_values_hook */ NULL,
     /* accept_focus */ NULL,
     /* intrinsics version */ XtVersion,
     /* callback offsets */ NULL,
     /* tm_table */ translations,
     /* query_geometry */ NULL,
     /* display_accelerator */ NULL,
     /* extension */ NULL
     }
};

WidgetClass xlCoreWidgetClass = (WidgetClass) & xlCoreClassRec;

/* ------------------------------------------------------------------- */
static void
ClassInitialize(void)
{
    InitXLImageProcTable();
}

static void
Initialize(Widget req, Widget new, ArgList args, Cardinal *num)
{
    XLCoreWidget cw = (XLCoreWidget) new;
    CW.image = NULL;

    CW.gc = DefaultGC(XtDisplay(cw), XScreenNumberOfScreen(XtScreen(cw)));
    InitFont(XtDisplay(cw), CW.gc, CW.font);
}

static void
Destroy(Widget widget)
{
    XLCoreWidget cw = (XLCoreWidget) widget;

    if (CW.image)
	XLImageDestroy(CW.image);
}

static void
Realize(Widget widget, XtValueMask *value_mask, XSetWindowAttributes *attr)
{
    XLCoreWidget cw = (XLCoreWidget) widget;
    Display *dpy = XtDisplay(widget);
    Window win;

    XtCreateWindow(widget, (u_int) InputOutput,
		   (Visual *) CopyFromParent, *value_mask, attr);
    win = XtWindow(widget);

    CW.image = XLImageCreate(dpy, win, CORE_WIDTH, CORE_HEIGHT);

    XtVaSetValues(widget, XtNbackground, 0, NULL);
}

static void
Redisplay(Widget widget, XEvent *ev, Region region)
{
    fprintf(stderr, "expose\n");
    if (XtIsRealized(widget)) {
	XLCoreWidget cw = (XLCoreWidget) widget;
	// ꡼κ
	FlushWindow();
	redraw_text();
    }
}

static void
Resize(Widget widget)
{
    if (XtIsRealized(widget)) {
	XLCoreWidget cw = (XLCoreWidget) widget;
	// եåȤѹ
    }
}

static void
Motion(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    XLCoreWidget cw = (XLCoreWidget) widget;

    {
	int x, y;
	XMotionEvent *me = (XMotionEvent *) ev;
	text_coordinate(me->x, me->y, &x, &y);
	if (alCore.text.attr[x][y] != alCore.text.highlight) {
	    alCore.text.highlight = alCore.text.attr[x][y];
	    fprintf(stderr, "<%d>\n", alCore.text.highlight);
	    redraw_text();
	}
    }

}

static void
ClickLeft(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
    fprintf(stderr, "tapped\n");
    alCore.dev.tapped = True;
}

static void
ClickRight(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
}

static void
CursorUp(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
}

static void
CursorDown(Widget widget, XEvent *ev, String *p, Cardinal *n)
{
}

/* 
 *  VRAM 
 */
void
FlushWindow()
{
    XLCoreWidget cw = (XLCoreWidget) alCore.x11_data.core;
    set_ximage(alCore.vram, CW.image, CORE_WIDTH, CORE_HEIGHT);
    XLImagePut(CW.image);

    XSync(XtDisplay((Widget) cw), False);
}

void
ClearWindow()
{
    XLCoreWidget cw = (XLCoreWidget) alCore.x11_data.core;
    XLImagePut(CW.image);
    XSync(Disp, False);
}

void
DrawChar(int ch, int x, int y, int attr)
{
    PutChar(Win, ch, x, y, attr);
    XSync(Disp, False);
}

/* 
 * ٥Ƚ
 */
void
Flip()
{
    XLCoreWidget cw = (XLCoreWidget) alCore.x11_data.core;

    XSync(XtDisplay((Widget) cw), False);

    while (!XtIsRealized((Widget) cw) || XtAppPending(alCore.x11_data.app)) {
	XEvent event;
	XtAppNextEvent(alCore.x11_data.app, &event);
	XtDispatchEvent(&event);
    }

    if (XtAppGetExitFlag(alCore.x11_data.app)) {
	LongJump(JUMP_END);
    }
}
