/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */
#ifndef __INCLUDE_XLADDER_H
#define __INCLUDE_XLADDER_H

#ifndef True
#define True 1
#endif

#ifndef False
#define False 0
#endif

/* 
 * ܤԤ
 */
#define WAIT_TIME 20000

/* 
 * ֽ
 */
#define ARC_SCN 0
#define ARC_BMP 1
#define ARC_WAV 2
#define ARC_ERR 3

#define FNAME_LEN 16

/* 
 * 
 */
#define ARG_LEN 64
#define ARG_NUM 7

#define FLAG_LEN 16

/* 
 * κĿ
 */
#define ALT_NUM 2

/* 
 * ץȽѥХåեΥ
 */
#define SCR_BUF_LEN 1024

/* 
 * եåط
 */
#define WIDTH 640
#define HEIGHT 480

/* 
 * ƥɽط
 */
#define TEXT_WIDTH 25
#define TEXT_HEIGHT 16

#define TEXT_WINDOW_POS 13

#define TEXT_WINDOW_TOP 128
#define TEXT_WINDOW_BORDER_TOP 144

#define BACKLOG_LEN 48

#define ATTR_NORMAL_MASK 0
#define ATTR_DICT_MASK (1 << 14)
#define ATTR_ALTN_MASK (1 << 15)
#define ATTR_ALL_MASK 0xffffffff

#define ATTR_NORMAL 0
#define ATTR_LINK 1
#define ATTR_ALTN 2
#define ATTR_LINK_HL 3
#define ATTR_ALTN_HL 4

/* 
 * 
 */
#define STAT_STACK_LEN 16

#define STAT_EMPTY 0
#define STAT_TEXT 1
#define STAT_ALTN 2
#define STAT_COMMAND 3
#define STAT_WAIT 4
#define STAT_DIALOG 5
#define STAT_DICT 6
#define STAT_BACKLOG 7

#ifdef DEBUG
#define dprintf(a) fprintf a
#else
#define dprintf(a)
#endif

#include <stdio.h>
#include <sys/types.h>
#include <setjmp.h>

#ifdef USE_X11
#include <X11/Intrinsic.h>
#endif

typedef struct {
    char name[FNAME_LEN];
    int pos;
    int width;
    int height;
    int osize;
    int esize;
} ArchiveItem;

typedef struct {
    FILE *fp;
    int num;
    int type;
    ArchiveItem *item;
} Archive;


typedef struct _SceneList {
    u_char name[ARG_LEN];
    int offset;
    struct _SceneList *next;
} SceneList;

typedef struct _FlagList {
    u_char value[FLAG_LEN];
    struct _SceneList *next;
} FlagList;

typedef struct {
  char *name;
  u_char *rgbmap; /* RGB ¤ӤΥӥåȥޥå */
  int width;
  int height;
} Image;

typedef enum {
        JUMP_NONE,
        JUMP_RESTART, 
        JUMP_TITLE,
        JUMP_END
} JumpParameter;

typedef struct {
    struct {
	u_char *data;
	int cur_pos;
	int end_pos;
	SceneList *scene_list;
    } script;

  struct {
    char *save_dir;
    char *app_dir;
  } dir;

    struct {
	int string[ARG_LEN];
	u_char label[ARG_LEN];
    } alternative[ALT_NUM];
    int alternative_num;

    FlagList *flag;
    FlagList *cg_flag;

    struct {
	int tvram[TEXT_WIDTH][TEXT_HEIGHT];
	int attr[TEXT_WIDTH][TEXT_HEIGHT];
	int x;
	int y;
      int clear;
      int highlight;
      int fullscreen;
    } text;

    struct {
	int tvram[TEXT_WIDTH][BACKLOG_LEN];
	int y;
	int loop;
    } text_history;

    struct {
	int per_char;
	int per_sentence;
	int per_page;
    } wait;

  struct {
    char bg[ARG_LEN];
    char left[ARG_LEN];
    char middle[ARG_LEN];
    char right[ARG_LEN];
    int pos;
  } scene;

  struct {
    int tapped;
  } dev;

  int brightness;
    int state_stack_depth;
    int state_stack[STAT_STACK_LEN];

  Image *work;
  Image *sub;
  Image *vram;

#ifdef USE_X11
  struct {
    Widget top, core;
    XtAppContext app;
  } x11_data;
#endif

    Archive *cg_arc;
    Archive *script_arc;
    Archive *sound_arc;

  jmp_buf jump;

} ALCore;

#ifdef MAIN
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN ALCore alCore;

u_char *get_text(Archive *arc, const char *name);
Archive *open_archive(char *filename);
void close_archive(Archive *arc);

void load_script(const char *name);
void free_script(void);

void add_flag(const char *flag, FlagList ** flist);
void clear_flag(FlagList ** flist);

int parse_arg(const char *cmd, char argv[ARG_NUM][ARG_LEN]);

/* file.c */
void save_persistent_info();
void load_persistent_info();
void save_game();
void load_game();

/* image.c */
void ImageInit();
Image *ImageCreate(int width, int height);
void ImageDestroy(Image *image);
void ImageLoad(Image *image, const char *name);
void ImageFill(Image *image, int x, int y, int w, int h, u_char r, u_char g, u_char b);
void ImageDisplay(int brightness);

#endif
