/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include <stdio.h>
#include <sys/types.h>

#include "aladder.h"

#define DICT_ENTRY_NUM 1024

#if 0
static int text_buf[SCR_BUF_LEN];
static int attr_buf[SCR_BUF_LEN];
static int text_buf_pos;
static int text_buf_len;
#endif

typedef struct _DictEntry {
    u_int *word;
    int id;
    int word_len;
    u_int *desc;
    struct _DictEntry *next;
} DictEntry;

DictEntry *dict_index[DICT_ENTRY_NUM];

void
dict_init(const char *dictfile)
{
    FILE *fp;
    int i, d = 0, id = 0;
    u_char sjis[BUFSIZ];
    u_char euc[BUFSIZ];
    u_char word[BUFSIZ];
    u_char desc[BUFSIZ];

    for (i = 0; i < DICT_ENTRY_NUM; i++) {
	dict_index[i] = NULL;
    }

    fp = fopen(dictfile, "r");
    if (fp == NULL) {
	fprintf(stderr, "򳫤ޤ\n");
	exit(1);
    }

    while (fgets(sjis, BUFSIZ, fp) != NULL) {
	sjis2euc(sjis, euc);
	if (euc[0] == '[') {
	    if (d) {
		dict_add_entry(word, desc, id++);
		d = 0;
	    }
	    strcpy(word, euc);
	    desc[0] = '\0';
	} else {
	    strcat(desc, euc);
	    d = 1;
	}
    }
    if (d)
	dict_add_entry(word, desc, id++);

    fclose(fp);
}

void
dict_destory()
{
}

static void
dict_add_entry(u_char *word, u_char *desc, int id)
{
    int buf[BUFSIZ];
    u_char *p;
    int i, j;
    u_int idx;
    DictEntry *de;

    de = (DictEntry *) malloc(sizeof(DictEntry));

    de->id = id;

    i = 0;
    p = word + 1;
    while (*p != ']') {
	if (*p < 0x80) {
	    buf[i++] = *p++;
	} else {
	    buf[i++] = (*p << 8) | *(p + 1);
	    p += 2;
	}
    }
    buf[i] = 0;

    de->word = (int *) calloc(i + 1, sizeof(int));
    for (j = 0; j < i + 1; j++) {
	de->word[j] = buf[j];
    }
    de->word_len = i;

    /* desc */
    i = 0;
    p = desc;
    while (*p != '\0') {
	if (*p < 0x80) {
	    buf[i++] = *p++;
	} else {
	    buf[i++] = (*p << 8) | *(p + 1);
	    p += 2;
	}
    }
    buf[i] = 0;

    de->desc = (int *) calloc(i + 1, sizeof(int));
    for (j = 0; j < i + 1; j++) {
	de->desc[j] = buf[j];
    }

    idx = (de->word[0] * de->word[1]) % DICT_ENTRY_NUM;

    if (dict_index[idx] == NULL) {
	de->next = NULL;
	dict_index[idx] = de;
    } else if (de->word_len > dict_index[idx]->word_len) {
	de->next = dict_index[idx];
	dict_index[idx] = de;
    } else {
	DictEntry *p = dict_index[idx];
	while (1) {
	    if (p->next == NULL) {
		p->next = de;
		de->next = NULL;
		break;
	    }
	    if (de->word_len > p->next->word_len) {
		de->next = p->next;
		p->next = de;
		break;
	    }
	    p = p->next;
	}
    }
}

#ifdef DEBUG
static void
dict_print()
{
    int i;

    for (i = 0; i < DICT_ENTRY_NUM; i++) {
	fprintf(stderr, "%d: ", i);
	if (dict_index[i] == NULL) {
	    fprintf(stderr, "[empty]\n");
	} else {
	    DictEntry *p = dict_index[i];
	    fprintf(stderr, "\n");
	    while (p != NULL) {
		fprintf(stderr, "  [%d]:", p->word_len);
		euc_put_string(p->word);
		fprintf(stderr, "\n");
		p = p->next;
	    }
	}
    }
}
#endif

int
text_len(int *text)
{
    int i;

    for (i = 0; text[i] != 0; i++);

    return i;
}

int
text_cmp(int *text1, int *text2, int num)
{
    int i;

    for (i = 0; i < num; i++) {
	if (text1[i] != text2[i])
	    return 0;
    }
    return 1;
}

void
dict_lookup(u_int *text, u_int *attr)
{
    int len;
    int i, j;
    static int loop = 0;

    len = text_len(text);
    // fprintf(stderr, "text_len: %d\n", len);

    for (i = 0; i < len; i++)
	attr[i] = 0;

    for (i = 0; i < len - 1;) {
	int idx, match = False;
	DictEntry *p;

	idx = (text[i] * text[i + 1]) % DICT_ENTRY_NUM;
	p = dict_index[idx];
	// fprintf(stderr, "hash val=%d: ", idx);
	while (p != NULL) {
	    if (text_cmp(text + i, p->word, p->word_len)) {
		match = True;
		break;
	    }
	    p = p->next;
	}
	if (match) {
	    // fprintf(stderr, "match\n");
	    for (j = 0; j < p->word_len; j++)
		attr[i++] = loop << 16 | p->id | ATTR_DICT_MASK;
	    if (++loop > 0xff)
		loop = 0;
	} else {
	    // fprintf(stderr, "not match\n");
	    i++;
	}
    }
}

#if 0
int
main()
{
    u_char buf[BUFSIZ];

    dict_init();
    // dict_print();

    while (fgets(buf, BUFSIZ, stdin) != NULL) {
	set_text(buf);
	dict_lookup(text_buf, attr_buf);
	euc_put_string_attr(text_buf, attr_buf);
    }
}
#endif

void
euc_put_string(int *s)
{
    while (*s != 0) {
	if (*s < 0x80) {
	    u_char c = (u_char) *s++;
	    fputc(c, stderr);
	} else {
	    u_char u, l;
	    u = *s >> 8 & 0xff;
	    l = *s & 0xff;
	    s++;
	    fprintf(stderr, "%c%c", u, l);
	}
    }
}

void
euc_put_string_attr(int *s, int *a)
{
    int i = 0;

    while (s[i] != 0) {
	if (s[i] < 0x80) {
	    u_char c = (u_char) s[i];
	    if (a[i] != 0) {
		fputc('<', stderr);
		fputc(c, stderr);
		fputc('>', stderr);
	    } else {
		fputc(c, stderr);
	    }
	    i++;
	} else {
	    u_char u, l;
	    u = s[i] >> 8 & 0xff;
	    l = s[i] & 0xff;
	    if (a[i] != 0) {
		fprintf(stderr, "<%c%c>", u, l);
	    } else {
		fprintf(stderr, "%c%c", u, l);
	    }
	    i++;
	}
    }
}
