/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include <stdlib.h>

#include "aladder.h"

static u_char bri_map[21][256];
static u_char alpha_map[256][256][256];

void
ImageInit()
{
    int i, j, k;

    for (i = 0; i <= 20; i++) {
	for (j = 0; j < 256; j++) {
	   int r = j * i / 20;
	   bri_map[i][j] = (u_char)r;
	}
    }

    for (i = 0; i < 256; i++) {
      for (j = 0; j < 256; j++) {
	for (k = 0; k < 256; k++) {
	  int r = (i * k + j * (0xff - k)) / 0xff;
	  alpha_map[i][j][k] = (u_char)r;
	}
      }
    }
}

Image *
ImageCreate(int width, int height)
{
    Image *img;

    img = (Image *) malloc(sizeof(Image));
    img->width = width;
    img->height = height;
    if (width * height) {
      img->rgbmap = (u_char *) calloc(width * height * 3, sizeof(u_char));
    } else {
      img->rgbmap = NULL;
    }

    return img;
}

void
ImageDestroy(Image *image)
{
    free(image->rgbmap);
    free(image);
}

void
ImageLoad(Image *image, const char *name)
{
    if (image->rgbmap != NULL) {
	free(image->rgbmap);
    }
    get_image(alCore.cg_arc, name, image);
}

void
ImageFill(Image *image, int x, int y, int w, int h, u_char r, u_char g,
	  u_char b)
{
    int i, j, idx;

    for (i = 0; i < h; i++) {
	idx = (image->width * (y + i) + x) * 3;
	for (j = 0; j < w; j++) {
	    image->rgbmap[idx++] = r;
	    image->rgbmap[idx++] = g;
	    image->rgbmap[idx++] = b;
	}
    }
}

void
ImageAdd(Image *dst, Image *src, int xoff, int yoff)
{
  if (src->width == dst->width && src->height == dst->height &&
      xoff == 0 && yoff == 0) {
    memcpy(dst->rgbmap, src->rgbmap, src->width * src->height * 3);
  } else {
    int i, j;
  
    for (i = 0; i < src->height; i++) {
    u_char *p = dst->rgbmap + (dst->width * 3 + dst->width % 4) * i + xoff * 3;
    u_char *q = src->rgbmap + (src->width * 3 + src->width % 4) * i;
    for (j = 0; j < src->width * 3 && j  + xoff * 3 < dst->width * 3; j++) {
      *p++ = *q++;
    }
    }
  }
}

void
ImageMerge(Image *dst, Image *src1, Image *src2, int alpha)
{
  int i, j;

  if (dst->width != src1->width || dst->width != src2->width ||
      dst->height != src1->height || dst->height != src2->height) {
    fprintf(stderr, "ƱβޡǤޤ\n");
    return;
  }
  
  for (i = 0; i < src1->height; i++) {
    u_char *p = dst->rgbmap + (dst->width * 3 + dst->width % 4) * i;
    u_char *q = src1->rgbmap + (src1->width * 3 + src1->width % 4) * i;
    u_char *r = src2->rgbmap + (src2->width * 3 + src2->width % 4) * i;
    for (j = 0; j < src1->width * 3; j++) {
      *p++ = alpha_map[*q++][*r++][alpha];
    }
  }
}

void
ImageAddWithMask(Image *dst, Image *src, Image *mask, int xoff, int yoff)
{
  int i, j;
  
  for (i = 0; i < src->height; i++) {
    u_char *p = dst->rgbmap + (dst->width * 3 + dst->width % 4) * i + xoff * 3;
    u_char *q = src->rgbmap + (src->width * 3 + src->width % 4) * i;
    u_char *r = mask->rgbmap + (mask->width * 3 + mask->width % 4) * i;
    for (j = 0; j < src->width * 3 && j  + xoff * 3 < dst->width * 3; j++, p++, q++, r++) {
      *p = alpha_map[*p][*q][*r];
    }
  }
}

void
ImageDisplay(int brightness)
{
  if (alCore.text.fullscreen) {
    if (brightness == 20) {
	memcpy(alCore.vram->rgbmap, alCore.work->rgbmap,
	       WIDTH * HEIGHT * 3);
    } else {
	int i;
	for (i = 0; i < WIDTH * HEIGHT * 3; i++) {
	    alCore.vram->rgbmap[i] =
		bri_map[brightness][alCore.work->rgbmap[i]];
	}
    }
  } else {
    int i, j;
    memcpy(alCore.vram->rgbmap, alCore.work->rgbmap,
	   WIDTH * HEIGHT * 3);

    for (i = 0 ; i < WIDTH * TEXT_WINDOW_TOP * 3; i++) {
      alCore.vram->rgbmap[i] =
	bri_map[brightness][alCore.work->rgbmap[i]];
    }
    for (i = TEXT_WINDOW_TOP; i < TEXT_WINDOW_BORDER_TOP; i++) {
      int bri = brightness + ((i - TEXT_WINDOW_TOP) * (20 - brightness)) / (TEXT_WINDOW_BORDER_TOP - TEXT_WINDOW_TOP);
      int base = i * WIDTH * 3;
      for (j = 0; j < WIDTH * 3; j++) {
	alCore.vram->rgbmap[base + j] =
	  bri_map[bri][alCore.work->rgbmap[base + j]];
      }
    }
  }
}
