/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */
#include "aladder.h"

static int text_buf[SCR_BUF_LEN];
static int text_buf_pos;
static int text_buf_len;

static int attr_buf[SCR_BUF_LEN];

#define TEXT alCore.text
#define HISTORY alCore.text_history

void
newpage()
{
  static int prev_page_top = 0;
    TEXT.clear = True;
    alCore.scene.pos = prev_page_top;
    prev_page_top = alCore.script.cur_pos;
    fprintf(stderr, "<PAGE TOP: %d>\n", alCore.scene.pos);
}

int
newline()
{
  int ret = 0;
    TEXT.tvram[TEXT.x][TEXT.y] = 0;
    HISTORY.tvram[TEXT.x][HISTORY.y] = 0;

    TEXT.x = 0;

    if (++TEXT.y >= TEXT_HEIGHT) {
      newpage();
      push_stack(STAT_WAIT);
      ret = -1;
    }

    if (++HISTORY.y >= BACKLOG_LEN) {
	HISTORY.y = 0;
	HISTORY.loop = True;
    }
    return ret;
}

int
put_char(int c, int attr)
{
  int type = ATTR_NORMAL;
  int ret = 0;

    if (TEXT.clear) {
	int i, j;

	for (i = 0; i < TEXT_HEIGHT; i++) {
	    TEXT.tvram[0][i] = 0;
	    for (j = 0; j < TEXT_WIDTH; j++) {
	      TEXT.attr[j][i] = 0;
	    }
	}
	TEXT.x = TEXT.y = 0;
	if (!TEXT.fullscreen) TEXT.y = TEXT_WINDOW_POS;

	ClearWindow();
	TEXT.clear = False;
    }

    if (c == 0xa) {
      if (newline() < 0) ret = -1;
      return ret;
    }

    if (attr & ATTR_DICT_MASK) {
      if (attr == TEXT.highlight) {
	type = ATTR_LINK_HL;
      } else {
	type = ATTR_LINK;
      }
    } else if (attr & ATTR_ALTN_MASK) {
      if (attr == TEXT.highlight) {
	type = ATTR_ALTN_HL;
      } else {
	type = ATTR_ALTN;
      }
    }

    DrawChar(c, TEXT.x, TEXT.y, type);

    TEXT.tvram[TEXT.x][TEXT.y] = c;
    TEXT.attr[TEXT.x][TEXT.y] = attr;
    HISTORY.tvram[TEXT.x][HISTORY.y] = c;
    TEXT.x++;

    if (TEXT.x >= TEXT_WIDTH - 1) {
      if (newline() < 0) ret = -1;
    }

    return ret;
}

void
debug_put_char(int c)
{
    u_char u, l;

    u = c >> 8 & 0xff;
    l = c & 0xff;

    if (c == 0xa) {
	fprintf(stderr, "\n");
	return;
    }

    fprintf(stderr, "%c%c", u, l);
}

void
set_text(const u_char *text)
{
    int i, j;

    for (i = 0, j = 0; j < strlen(text);) {
	if (text[j] < 0x80) {
	    text_buf[i++] = text[j];
	    j++;
	} else {
	    text_buf[i++] = text[j] << 8 | text[j + 1];
	    j += 2;
	}
    }
    text_buf_pos = 0;
    text_buf_len = i;

    dict_lookup(text_buf, attr_buf);
}

int
proc_text()
{
  int attr;

    if (alCore.wait.per_char) {
	put_char(text_buf[text_buf_pos], attr_buf[text_buf_pos]);
	text_buf_pos++;
	if (text_buf_pos >= text_buf_len) {
	    return 1;
	}
    } else {
      int i;
      for (; text_buf_pos < text_buf_len; text_buf_pos++) {
	if (put_char(text_buf[text_buf_pos], attr_buf[text_buf_pos]) < 0) {
	  fprintf(stderr, "[ڡդˤ]\n");
	  text_buf_pos++;
	  return 0;
	}
      }
      return 1;
    }
    return 0;
}

int
redraw_text()
{
    int i, j;
    int x, y;
    int type;

    x = TEXT.x;
    y = TEXT.y;

    for (i = 0; i < TEXT_HEIGHT; i++) {
	for (j = 0; j < TEXT_WIDTH; j++) {
	    if (TEXT.tvram[j][i] == 0)
		break;
	    if (TEXT.attr[j][i] & ATTR_DICT_MASK) {
	      if (TEXT.attr[j][i] == TEXT.highlight) {
		type = ATTR_LINK_HL;
	      } else {
		type = ATTR_LINK;
	      }
	    } else if (TEXT.attr[j][i] & ATTR_ALTN_MASK) {
	      if (TEXT.attr[j][i] == TEXT.highlight) {
		type = ATTR_ALTN_HL;
	      } else {
		type = ATTR_ALTN;
	      }
	    } else {
	      type = ATTR_NORMAL;
	    }
	    DrawChar(TEXT.tvram[j][i], j, i, type);
	}
    }

    TEXT.x = x;
    TEXT.y = y;
}


#if 0
void
print_history()
{
    int x, y;

    fprintf(stderr, "\n*** history end ***\n");
    fprintf(stderr, "%d:%d\n", HISTORY.y, HISTORY.loop);
    if (HISTORY.loop) {
	for (y = HISTORY.y + 1; y < BACKLOG_LEN; y++) {
	    x = 0;
	    while (HISTORY.tvram[x][y] != 0) {
		debug_put_char(HISTORY.tvram[x++][y]);
	    }
	    fprintf(stderr, "\n");
	}
	for (y = 0; y < HISTORY.y; y++) {
	    x = 0;
	    while (HISTORY.tvram[x][y] != 0) {
		debug_put_char(HISTORY.tvram[x++][y]);
	    }
	    fprintf(stderr, "\n");
	}
    } else {
	for (y = 0; y < HISTORY.y; y++) {
	    x = 0;
	    while (HISTORY.tvram[x][y] != 0) {
		debug_put_char(HISTORY.tvram[x++][y]);
	    }
	    fprintf(stderr, "\n");
	}
    }

    fprintf(stderr, "\n*** history end ***\n");
}
#endif
